/********************************************************
 * graph -- perform bitmapped graphs on the screen.	*
 *							*
 * Programming problem 11-1, 11-2, 11-3.			*
 *							*
 * This program was based on Example 11-2 in the book.	*
 * (Note: Early editions of the book incorrectly	*
 * gave Example 11-1 as the example to use.)		*
 *							*
 * Usage:						*
 *	Run the program.				*
 *	Results are displayed				*
 ********************************************************/
#include <iostream.h>

const int X_SIZE = 32; // size of array in the X direction 
const int Y_SIZE = 32; // size of the array in Y direction 
/*
 * We use X_SIZE/8 since we pack 8 bits per byte
 */
char graphics[X_SIZE / 8][Y_SIZE];   // the graphics data 

/********************************************************
 * set_bit -- set a bit in the graphics array.		*
 *							*
 * Parameters 						*
 *	x,y -- location of the bit.			*
 ********************************************************/
inline void set_bit(const int x,const int y)
{
    graphics[(x)/8][y] |= (0x80 >>((x)%8));
}
/********************************************************
 * clear_bit -- clear a bit in the graphics array.	*
 *							*
 * Parameters 						*
 *	x,y -- location of the bit.			*
 ********************************************************/
inline void clear_bit(const int x,const int y)
{
    graphics[(x)/8][y] &= ~ (0x80 >>((x)%8));
}
/********************************************************
 * test_bit -- test a bit in the graphics array.	*
 *							*
 * Parameters 						*
 *	x,y -- location of the bit.			*
 *							*
 * Returns						*
 *	1 -- bit set					*
 *	0 -- bit clear					*
 *							*
 * Note: Bool is not used because it's not in enough	*
 * compilers to make this code portable.		*
 ********************************************************/
inline int test_bit(const int x,const int y)
{
    return ((graphics[(x)/8][y] &= (0x80 >>((x)%8))) != 0);
}

main()
{
    int   x_loc, y_loc;        // current location we are setting
    int   loc;			// Loc for drawing line
    void  print_graphics(void); // print the data

    /* 
     * Set the entire array to white
     */
    for (x_loc = 0; x_loc < X_SIZE; x_loc++) {
	for (y_loc = 0; y_loc < Y_SIZE; y_loc++) {
	   clear_bit(x_loc, y_loc);
	}
    }
    cout << "------ Programming Problem 11-2\n";
    /* Draw top/bottom of the square */
    for (x_loc = 1; x_loc <= 10; x_loc++) {
       set_bit(x_loc, 1);	// Top
       set_bit(x_loc, 10);	// Bottom
    }
    // Draw sides of the square
    for (y_loc = 1; y_loc <= 10; y_loc++) {
       set_bit(1, y_loc);	// Left
       set_bit(10, y_loc);	// Right
    }
    print_graphics();

    cout << "Press return to continue: ";
    char continue_line[1];	// Line for continue
    cin.getline(continue_line, sizeof(continue_line));

    cout << "------ Programming Problem 11-3\n";
    /* 
     * Set the entire array to black
     */
    for (x_loc = 0; x_loc < X_SIZE; x_loc++) {
	for (y_loc = 0; y_loc < Y_SIZE; y_loc++) {
	   set_bit(x_loc, y_loc);
	}
    }
    for (loc = 0; loc < X_SIZE; ++loc)
        clear_bit(loc, loc);

    print_graphics();
    return (0);
}
/********************************************************
 * print_graphics -- print the graphics bit array       *
 *              as a set of X and .'s.                  *
 ********************************************************/
void print_graphics(void)
{
    int x;     // current x BYTE 
    int y;     // current y location 
    int bit;   // bit we are testing in the current byte 

    for (y = 0; y < Y_SIZE; ++y) {

        // Loop for each byte in the array 
        for (x = 0; x < X_SIZE / 8; ++x) {

            // Handle each bit 
            for (bit = 0x80; bit > 0; bit = (bit >> 1)) {
                if ((graphics[x][y] & bit) != 0)
                    cout << 'X';
                else
                    cout << '.';
            }
        }
        cout << '\n';
    }
}
