/********************************************************
 * flash -- flash card program.				*
 *							* 
 * Programming Problem: 13-5				*
 *							*
 * Usage:						*
 *	Run the program.				*
 * 	It will type out an English			*
 *	Type in the corresponding French word.		*
 *	Repeat till you get them all right.		*
 *							*
 * Limitations:						*
 *	We don't handle special characters or accents	*
 *	The word table is built in.			*
 *	The language is built in.			*
 ********************************************************/
#include <iostream.h>

struct single_card {
    char question[40];	// English word (the question)
    char answer[40];	// French word (the answer)
    int  good;		// True if we've answered this one correctly
};

/*
 * Table of cards to use.
 *	The French words have yet to be filled in since I don't know
 *	French.
 */
struct single_card cards[] = {
   {"one", 	"A"},
   {"two",	"B"},
   {"three",	"C"},
   {"",		""}
};

/********************************************************
 * flash -- flash card class.				*
 *							*
 * Gives the next card to the caller and waits for 	*
 * right or wrong to be called.  If right, it card	*
 * will be removed from the "todo" list.		*
 *							*
 * Member functions:					*
 *	flash_card(single_card cards[])			*
 *		-- initialize the card class		*
 *							*
 *	single_card get_card()				*
 *		-- Get the next card from the deck	*
 *							*
 *	right -- Called when the last card was right	*
 *	wrong -- Called when the last card was wrong	*
 *	done -- If true, no more cards.			*
 *							*
 * Note: Implementation of this would be much simpler	*
 *	if we could use pointers.			*
 ********************************************************/
const int CARD_MAX = 30;		// Biggest stack of cards

class flash_card {
    private:
	single_card cards[CARD_MAX];	// My list of cards
	unsigned int n_cards;		// Number of cards
	int right_list[CARD_MAX];	// True if cards are right
	unsigned int cur_card;		// Current card
    public:
	flash_card(const single_card init_cards[]);
	// Destructor defaults
	// Copy constructor defaults
	// Assignment operator defaults
    public:
	// Get the next card
	const single_card &get_card(void);

	// Card is right
	void right(void) {
	    right_list[cur_card] = 1;
	}

	// Card is wrong (do nothing)
	void wrong(void) {}
	    
	int done(void);	// Return true if we're done
	
};
/********************************************************
 * flash_card::flash_card -- initialize the flash 	*
 *	card list.					*
 ********************************************************/
flash_card::flash_card(
    const single_card init_cards[]
) {
    for (n_cards = 0; 
	 init_cards[n_cards].question[0] != '\0';
	 ++n_cards) {
	cards[n_cards] = init_cards[n_cards];
	right_list[n_cards] = 0;
    }
    cur_card = n_cards-1;
}
/********************************************************
 * flash_card::done -- Returns true if we're done	*
 ********************************************************/
int flash_card::done(void)
{
    unsigned int i;	// Checking index

    for (i = 0; i < n_cards; ++i) {
	if (right_list[i] == 0)
	    return (0);
    }

    return (1);
}
/********************************************************
 * flash_card::get_card -- return the next unused card	*
 ********************************************************/
const single_card &flash_card::get_card(void)
{
    unsigned int i;	// Index for going through cards

    // Test for no unused cards
    if (done())
       return (cards[0]);

    // Move to the next card
    ++cur_card;
    cur_card %= n_cards;

    for (i = 0; i < n_cards; ++i) {
        if (right_list[cur_card] == 0)
	    return (cards[cur_card]);

	// Move to the next card
	++cur_card;
	cur_card %= n_cards;
    }
    // We'll never get here
    cout << "Internal error: get_card reached end\n";
    return (cards[0]);
}

main()
{
    flash_card test_cards(cards);

    while (!test_cards.done()) {
	const single_card &cur_card = test_cards.get_card();
	char answer[100];	// Answer from the user

	cout << "English: " << cur_card.question << " French? ";
	cin.getline(answer, sizeof(answer));

	if (strcmp(answer, cur_card.answer) == 0)
	    test_cards.right();
	else
	    test_cards.wrong();
    }
    return (0);
}
