/********************************************************
 * fract -- a fraction handing class			*
 *							*
 * Programming problem: 18-1.				*
 *							*
 * Usage:						*
 *	Run the program to test the class.		*
 ********************************************************/
#include <iostream.h>


/********************************************************
 * fract -- a fraction class.				*
 *							*
 * A fraction is defined as   top/bottom.		*
 *							*
 * This class defines the following operators:		*
 *							*
 *	f = f + f;					*
 *	f = f - f;					*
 *	f = f * f;					*
 *	f = f / f;					*
 *							*
 *	f += f;						*
 *	f -= f;						*
 *	f *= f;						*
 *	f /= f;						*
 *							*
 *	f == f						*
 *	f <= f						*
 *	f <  f						*
 *	f >  f						*
 *	f >= f						*
 *	f != f						*
 *							*
 *	f++						*
 *	++f						*
 *	f--						*
 *	--f						*
 *							*
 *	out_file << f					*
 *	in_file >> f					*
 *							*
 * Member functions:					*
 *	set(int i1, int i2)				*
 *		-- Set fraction to  i1/i2		*
 ********************************************************/
class fract {
    private:
	int top;	// Top part of fraction
	int bottom;	// Bottom part of the fraction
     
    private:
	void simplify(void); // Simplify a fraction

    public: 
	// Default constructor
	fract(void) {
	    top = bottom = 0;
	}
	// Construct fraction out of two integers
	fract(const int i_top, const int i_bottom) {
	    top = i_top;
	    bottom = i_bottom;
	    simplify();
	}
	// Destructor defaults
	// Copy constructor defaults
	// Assignment operator defaults

	// Returns the top part of the fraction
	int get_top(void) const { 
	    return (top);
	}
	// Returns the bottom part of the fraction
	int get_bottom(void) const { 
	    return (bottom);
	}

	// -------------- Operators ---------------------
        //	f = f + f;
	fract operator + (const fract &oper2) {
	    fract result(
		top * oper2.bottom + oper2.top * bottom,
		bottom * oper2.bottom);
	    return (result);
	}
        //	f = f - f;
	fract operator - (const fract &oper2) {
	    fract result(
		top * oper2.bottom - oper2.top * bottom,
		bottom * oper2.bottom);
	    return (result);
	}
        //	f = f * f
	fract operator * (const fract &oper2) {
	    fract result(
		top * oper2.top,
		bottom * oper2.bottom);
	    return (result);
	}
        //	f = f / f;
	fract operator / (const fract &oper2) {
	    fract result(
		top * oper2.bottom,
		bottom * oper2.top);
	    return (result);
	}
        //	f += f
	fract &operator += (const fract &oper1) {
	    int new_top = top * oper1.bottom + oper1.top * bottom;
	    int new_bottom = bottom * oper1.bottom;

	    top = new_top;
	    bottom = new_bottom;
	    simplify();
	    return (*this);
	}
        //	f -= f;
	fract &operator -= (const fract &oper1) {
	    int new_top = top * oper1.bottom - oper1.top * bottom;
	    int new_bottom = bottom * oper1.bottom;

	    top = new_top;
	    bottom = new_bottom;
	    simplify();
	    return (*this);
	}
        //	f *= f;
	fract &operator *= (const fract &oper1) {
	    top *= oper1.top;
	    bottom *= oper1.bottom;
	    simplify();
	    return (*this);
	}
        //	f /= f;
	fract &operator /= (const fract &oper1) {
	    top *= oper1.bottom;
	    bottom *= oper1.top;
	    simplify();
	    return (*this);
	}
        //	f == f
	int operator ==(const fract &oper2) const {
	    return ((top == oper2.top) && (bottom == oper2.bottom));
	}
        //	f <= f
	int operator <=(const fract &oper2) const {
	    int top1 = top * oper2.bottom;	// Top of normalized fract1
	    int top2 = oper2.top * bottom;	// Top of normalized fract2
	    return (top1 <= top2);
	}
        //	f <  f
	int operator <(const fract &oper2) const {
	    int top1 = top * oper2.bottom;	// Top of normalized fract1
	    int top2 = oper2.top * bottom;	// Top of normalized fract2
	    return (top1 < top2);
	}
        //	f >  f
	int operator >(const fract &oper2) const {
	    int top1 = top * oper2.bottom;	// Top of normalized fract1
	    int top2 = oper2.top * bottom;	// Top of normalized fract2
	    return (top1 > top2);
	}
        //	f >= f	
	int operator >=(const fract &oper2) const {
	    int top1 = top * oper2.bottom;	// Top of normalized fract1
	    int top2 = oper2.top * bottom;	// Top of normalized fract2
	    return (top1 >= top2);
	}
        //	f != f
	int operator !=(const fract &oper2) const {
	    return ((top != oper2.top) || (bottom != oper2.bottom));
	}
        //	f++				
	fract operator ++(int) {
	    fract result(*this);	// Result before ++
	    top += bottom;
	    return (result);
	}
        //	++f
	fract operator ++(void) {
	    top += bottom;
	    return (*this);
	}
        //	f--
	fract operator --(int) {
	    fract result(*this);	// Result before ++
	    top -= bottom;
	    return (result);
	}
        //	--f	
	fract operator --(void) {
	    top -= bottom;
	    return (*this);
	}
        //	in_file >> f
	istream &operator >> (istream &file);
};

/********************************************************
 *  out_file << f -- output a fraction			*
 ********************************************************/
ostream &operator << (
    ostream &file,	// File to write to 
    const fract &f	// Fraction to write
) {
    file << f.get_top() << '/' << f.get_bottom();
    return (file);
}
/********************************************************
 * istream >> fract -- read a fraction 			*
 *							*
 * Returns						*
 *	reference to the input file			*
 ********************************************************/
istream &fract::operator >> (
   istream &in_file 	// Input file
)
{
    char ch;		// Random character used to verify input

    in_file.ipfx(1);	// Tell the i/o system we are reading formatted
    in_file >> ws;	// Skip whitespace

    if (in_file.bad()) return (in_file);	

    in_file >> top;

    if (in_file.bad()) return (in_file);

    in_file >> ws >> ch;	// Get first character after number

    if (in_file.bad()) return (in_file);

    if (ch != '/') {
       in_file.setf(ios::failbit);
       return (in_file);
    }

    in_file >> bottom;
    if (in_file.bad()) return (in_file);
    return (in_file);
}
/********************************************************
 * simplify -- simplify a fraction.			*
 *							*
 * Find the gcd (greatest common divisor) that goes	*
 * into both top and bottom and divide them by it.	*
 *							*
 * Note: There is a very quick and clever algorithm	*
 * that handles finding the GSD.  We don't use it.	*
 ********************************************************/
void fract::simplify(void)
{
    int test;	// Test gsd
    int gsd = 1;	// Gsd we've found so far

    for (test = 2; /* below */; ++test) {
	// See if we've reached the end
	if ((test * test) > top)
	    break;

	if ((test * test) > bottom)
	    break;

	if (((bottom % test) == 0) &&
	    ((top % test) == 0))
	    gsd = test;
    }
    top /= gsd;
    bottom /= gsd;
}

main()
{
    fract half(1, 2);	// The fraction 1/2
    fract third(1, 3);	// The fraction 1/3

    cout << (half + third) << " = " << half << " + " << third << '\n';
    cout << (half - third) << " = " << half << " - " << third << '\n';
    cout << (half * third) << " = " << half << " * " << third << '\n';
    cout << (half / third) << " = " << half << " / " << third << '\n';
    cout << '\n';
    
    fract work;
    work = half;

    work = half;
    work += third;
    cout << (work) << " = " << half << " += " << third << '\n';

    work = half;
    work -= third;
    cout << (work) << " = " << half << " -= " << third << '\n';

    work = half;
    work *= third;
    cout << (work) << " = " << half << " *= " << third << '\n';

    work = half;
    work /= third;
    cout << (work) << " = " << half << " /= " << third << '\n';

    cout << '\n';

    cout << (half == third) << " = " << half << " == " << third << '\n';
    cout << (half <= third) << " = " << half << " <= " << third << '\n';
    cout << (half >  third) << " = " << half << " > " << third << '\n';
    cout << (half <  third) << " = " << half << " < " << third << '\n';
    cout << (half <= third) << " = " << half << " <= " << third << '\n';
    cout << (half != third) << " = " << half << " != " << third << '\n';

    cout << '\n';

    work = half;
    --work;
    cout << (work) << " = " << half << "++\n";

    work = half;
    --work;
    cout << (work) << " = " << half << "--\n";

    work = half;
    work++;
    cout << (work) << " = ++" << half << "\n";

    work = half;
    work--;
    cout << (work) << " = --" << half << "\n";

    return (0);
}
