/********************************************************
 * linked list class.					*
 *							*
 * From the "linked.h" file in the programs from 	*
 * Practical C++.					*
 ********************************************************/

const int COUNT_MAX = 8;	// Max number of element to print on a line

// Define an element of a linked list
class linked_list_element {
    public:
	int data;		// Data for the element

        linked_list_element(const int i_data) {
	    data = i_data;
	    next_ptr = NULL;
	}
	// Destructor defaults
    private:
	// No copy constructor
        linked_list_element(const linked_list_element &);
	
	// No assignment operator
        linked_list_element operator =(const linked_list_element &);

    private:
	// Pointer to the next element
	class linked_list_element *next_ptr;
	// Pointer to the previous element
	class linked_list_element *prev_ptr;
    
    friend class linked_list;	// The list itself
};

// The linked list
class linked_list {
    private:
  	linked_list_element *first_ptr;	// First element of the list
  	linked_list_element *last_ptr;	// Last element of the list

    public:
	// Create the list
	linked_list(void) { 
	    first_ptr = NULL; 
	    last_ptr = NULL;
	}
	~linked_list(void)	// Destroy the list
	{
	    while (first_ptr != NULL) {
		// Pointer to the next item in the list
	        linked_list_element *next_ptr = first_ptr->next_ptr;

		delete first_ptr;
		first_ptr = next_ptr;
	    }
	}

    private:
	// Don't allow use of the copy constructor
	linked_list(const linked_list &);	

    public:
	// Find an element in the list
	void add(int data) {
	    // The element we are adding
	    linked_list_element *new_element = new linked_list_element(data);
	    if (first_ptr == NULL) {
	        first_ptr = new_element;
		last_ptr = new_element;
		new_element->next_ptr = NULL;
		new_element->prev_ptr = NULL;
	    } else {
		last_ptr->next_ptr = new_element;
		new_element->prev_ptr = last_ptr;
		last_ptr = new_element;
	    }
	}

	// Delete an item from the list
	void remove(const int data) {
	    linked_list_element *cur_ptr;	// Current item
	    linked_list_element *prev_ptr;	// Previous item

	    prev_ptr = NULL;
	    for (cur_ptr = first_ptr; 
	         cur_ptr != NULL; 
		 cur_ptr = cur_ptr->next_ptr) {
	        if (cur_ptr->data == data) {
		    // Are we deleting the first element
		    if (cur_ptr == first_ptr) {
		        first_ptr = cur_ptr->next_ptr;
			cur_ptr->prev_ptr = NULL;

		    // Are we deleting the last element
		    } else if (cur_ptr == last_ptr) {
		        last_ptr = prev_ptr;
			last_ptr->next_ptr = NULL;
		    } 
		    
		    if (cur_ptr->next_ptr != NULL)
		        cur_ptr->next_ptr->prev_ptr = cur_ptr->prev_ptr;

		    if (cur_ptr->prev_ptr != NULL)
		        cur_ptr->prev_ptr->next_ptr = cur_ptr->next_ptr;

		    delete cur_ptr;
		    cur_ptr = NULL;
		    return;
		}
		prev_ptr = cur_ptr;
	    }
	    // At this point we didn't find the data.
	    // So don't worry about deleting what's not there
	}
	     
	    

	void print(ostream &out_file) {
	    // Element we are printing
	    linked_list_element *cur_element;
	    int count = 0;	// Number of elements on this line 

	    for (cur_element = first_ptr; 
	         cur_element != NULL; 
		 cur_element = cur_element->next_ptr) {
		if (count == COUNT_MAX) {
		    count = 0;
		    out_file << '\n';
		}
	        out_file << setw(5) << cur_element->data << "   ";
		++count;
	    }
	    out_file << '\n';
	}
};
