/********************************************************
 * xref -- cross reference generator.			*
 *							*
 * Programming Problem: 20-1				*
 *							*
 * Based on the Words program from Practical C++	*
 *                                                      *
 * Usage:                                               *
 *      xref <file>                                    *
 ********************************************************/
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

#include "linked.h"

class tree {
    private:
	// The basic node of a tree
	class node {
	    private:
		node    *right;      // tree to the right 
		node    *left;       // tree to the left 
	    public:
		const char    *word; // word for this tree 
		linked_list   line_list;// List of line references

	    friend class tree;	    
	};

	// the top of the tree 
	node *root;

	// Enter a new node into a tree or sub-tree
	void enter_one(
	   node *&node, 		// Node of sub-tree to look at
	   const char *const word,	// Word to add
	   const int line_number	// Where it's referenced
	);

	// Print a single node
	void print_one(node *top);
    public:
	tree(void) { root = NULL;}

	// Add a new word to our tree
        void enter(const char *const word, const int line_number) {
	    enter_one(root, word, line_number);
	}

	// Print the tree
	void print(void) {
	    print_one(root);
	}
};

static tree xref;	// List of words we are looking for
	
/********************************************************
 * get1 -- get a single character, echo it, line number	*
 *		handling as well.			*
 *							*
 * Returns						*
 *	Character gotten or EOF if end of file.		*
 ********************************************************/
int get1(
    istream &in_file,	// File we are reading
    int &line_number	// Current line number
) {
    int ch;		// Character we are getting
    static int line_out = 1;	// Output line number before starting

    ch = in_file.get();
    if (ch == EOF)
        return(ch);

    if (line_out) {
        ++line_number;
        cout << setw(4) << line_number << ' ';
	line_out = 0;
    }
    line_out = (ch == '\n');
    cout << char(ch);
    return (ch);
}


/********************************************************
 * scan -- scan the file for words                      *
 *                                                      *
 * Parameters                                           *
 *      name -- name of the file to scan                *
 ********************************************************/
void scan(char *name)
{
    char word[100];     // word we are working on 
    unsigned int  index;// index into the word 
    int  ch;            // current character 
    ifstream in_file;   // input file 
    int line_number = 0; // Current line number

    in_file.open(name, ios::in);
    if (in_file.bad()) {
        cerr << "Error:Unable to open " << name << '\n';
        exit(8);
    }
    while (1) {
        // scan past the whitespace 
        while (1) {
            ch = get1(in_file, line_number);

            if (isalpha(ch) || (ch == EOF))
                break;
        }

        if (ch == EOF)
            break;

        word[0] = ch;
        for (index = 1; index < sizeof(word); ++index) {
            ch = get1(in_file, line_number);
            if (!isalpha(ch))
                break;
            word[index] = ch;
        }
        // put a null on the end 
        word[index] = '\0';

        xref.enter(word, line_number);
    }
}



/********************************************************
 * tree::enter_one -- enter a word into the tree        *
 *                                                      *
 * Parameters                                           *
 *      new_node -- current node we are looking at      *
 *      word -- word to enter                           *
 ********************************************************/
void tree::enter_one(
   node *&new_node, 		// Sub-tree to look at
   const char *const word,	// Word to add
   const int line_number	// Where referenced
)
{
    int  result;        // result of strcmp 

    // see if we have reached the end 
    if (new_node == NULL) {
        new_node = new node;

        new_node->left = NULL;
        new_node->right = NULL;
        new_node->word = strdup(word);
    }
    result = strcmp(new_node->word, word);
    if (result == 0) {
	new_node->line_list.add(line_number);
        return;
    }

    if (result < 0)
        enter_one(new_node->right, word, line_number);
    else
        enter_one(new_node->left, word, line_number);
}

/********************************************************
 * tree::print_one -- print out the words in a tree    	*
 *                                                      *
 * Parameters                                           *
 *      top -- the root of the tree to print            *
 ********************************************************/
void tree::print_one(node *top)
{
    if (top == NULL)
        return;                 // short tree 

    print_one(top->left);
    cout << setw(7) << top->word << ' ';
    top->line_list.print(cout);
    print_one(top->right);
}

main(int argc, char *argv[])
{
    if (argc != 2) {
        cerr <<  "Error:Wrong number of parameters\n";
        cerr <<  "      on the command line\n";
        cerr <<  "Usage is:\n";
        cerr <<  "    xref 'file'\n";
        exit(8);
    }
    scan(argv[1]);
    xref.print();
    return (0);
}
