/********************************************************
 * queue -- a simple queue class and test routine	*
 *		This one has exception handling		*
 *							*
 * Programming Problem: 22-1				*
 ********************************************************/
#include <iostream.h>
// Triggered by a bounds error
class bounds_error{
    public:
        char *reason;

	bounds_error(char *i_reason) {
	    reason = i_reason;
	}
	// Destructor defaults
	// Copy constructor defaults
	// Assignment operator defaults
};         
/********************************************************
 * Queue class.						*
 *	A queue or FIFO is a first in, first out data	*
 *	storage.					*
 * 							*
 * Member functions:					*
 *	put  -- puts an item in the queue		*
 *	get  -- gets an items from the queue		*
 ********************************************************/
const int QUEUE_SIZE = 100;	// Max number of elements in queue

class queue {
    private:
	unsigned short int get_index;	// Index for the next get
	unsigned short int put_index;	// Index for the next put
	int data[QUEUE_SIZE];		// Data in the queue
	int n_items;			// Number of items in the queue

	// Increments (and wraps) a queue index
	void inc_index(
	    unsigned short int &index		// Index to increment
	) {
	    ++index;		// Move up
	    index %= QUEUE_SIZE;// Wrap if needed
	}
    public:
	queue(void) {
	   put_index = 0;
	   get_index = QUEUE_SIZE-1;
	   n_items = 0;
	}
	// Destructor defaults
    private:
	queue(const queue &);	// No copy constructor

	// No assignment operator
	queue operator= (const queue &);
    public:
	// Adds an item to the queue
	void put(
	    int item		// Item to put on the queue
	) {
	    ++n_items;
	    if (n_items >= QUEUE_SIZE) throw(bounds_error("Overflow"));

	    data[put_index] = item;
	    inc_index(put_index);
	}

	// Returns the top item in the queue
	int get(void) {
	    --n_items;
	    if (n_items < 0) throw(bounds_error("Underflow"));

	    inc_index(get_index);
	    return (data[get_index]);
	}
};

main()
{
    queue test_queue;	// Queue to test

    test_queue.put(1);
    test_queue.put(2);
    test_queue.put(3);
    test_queue.put(4);

    cout << "Get returns " << test_queue.get() << '\n';
    cout << "Get returns " << test_queue.get() << '\n';
    cout << "Get returns " << test_queue.get() << '\n';
    cout << "Get returns " << test_queue.get() << '\n';

    try {
        int i;	// Random index

	for (i = 0; i < 200; i++)
	    test_queue.put(i);
    }
    catch (bounds_error &err) {
        cerr << "Bounds error: " << err.reason << '\n';
    }
    return (0);
}
