/********************************************************
 * set -- define a simple set template			*
 * 							*
 * Programming Problem: 24-3.				*
 *							*
 * Member functions:					*
 *	set -- set an element of the set		*
 *	clear -- remove an element from the set		*
 *	test -- test to see if an element is in the set	*
 *							*
 * Note: classes in the set must have an == operator	*
 * defined for them.					*
 ********************************************************/
#ifndef _SET_H_	/* Prevent double includes */
#define _SET_H_

class set_overflow {};	// Created when a set overflows

const int DEFAULT_SET_SIZE = 100;	// Default size of set if not given

template<class element>
class set_class {
    private:
        element *data;	// Data that's in the set
	int *used;	// True if an element is used
	const unsigned int set_size;	// Max number of elements
    public:
        set_class(int i_set_size = DEFAULT_SET_SIZE) : set_size(i_set_size) {
	    data = new element[set_size];
	    used = new int[set_size];
	    memset(used, '\0', sizeof(int)*set_size);
	}
	virtual ~set_class(void) {
	    delete []data;
	    data = NULL;
	    delete used;
	    used = NULL;
	}
    private:
        // No copy constructor
	set_class(const set_class &);

        // No assignment operator
	set_class operator =(const set_class &);
    public:
        void set(element item) {
	    unsigned int i;	// General purpose index

	    // Scan to see if the item is in the set
	    for (i = 0; i < set_size; ++i) {
	        if (used[i]) {
		    if (data[i] == item)
		        return;		// Already in the set
		}
	    }

	    // Scan for free element
	    for (i = 0; i < set_size; ++i) {
	        if (!used[i]) {
		    data[i] = item;
		    used[i] = 1;
		    return;
		}
	    }
	    // At this point we ran out of room
	    set_overflow error;	// Error information
	    throw(error);
	}
	void clear(element item) {
	    unsigned int i;	// General purpose index

	    // Scan to see if the item is in the set
	    for (i = 0; i < set_size; ++i) {
	        if (used[i]) {
		    if (data[i] == item) {
		        used[i] = 0;
		        return;		// Deleted item
		    }
		}
	    }
	    // It is not an error to clear an item that's
	    // not there
	}
	int test(element item) {
	    unsigned int i;	// General purpose index

	    // Scan to see if the item is in the set
	    for (i = 0; i < set_size; ++i) {
	        if (used[i]) {
		    if (data[i] == item) 
		        return (1);		// Item found
		}
	    }
	    return (0);		// Item not found
	}
};
#endif /* _SET_H_ */

