/********************************************************
 * io -- mailing list I/O module			*
 *							*
 * Functions defined:					*
 *	read_mail -- read in the mailing list		*
 *	write_mail -- write out the mailing list	*
 *							*
 * File format:						*
 *	#--magic string--				*
 *	Name						*
 *	Address1					*
 *	Address2					*
 *	City						*
 *	State						*
 *	Zip						*
 *	----		(End of field separator)	*
 ********************************************************/
#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>

#include "mail.h"
#include "io.h"

const char MAIL_MAGIC[] = "#Mail list file Version 1.0";
const char ENTRY_END[] = "---";
/********************************************************
 * read_mail -- read in the mailing list.		*
 *							*
 * Returns						*
 *	0 -- no error.					*
 *	NZ -- error occurred, message written.		*
 ********************************************************/
int read_mail(
    const char file_name[]	// Name of the file to read
)
{
    ifstream in_file(file_name);	// File to read

    if (in_file.bad()) {
	cerr << "Error: Could not read file " << file_name << '\n';
	return (-1);
    }

    char magic[100];	// Magic line
    in_file.getline(magic, sizeof(magic));

    if (strcmp(magic, MAIL_MAGIC) != 0) {
	cerr << "Error: Bad magic number in input\n";
	return (-2);
    }

    while (!in_file.eof()) {
	struct mail new_entry;	// Entry we are creating

	in_file.getline(new_entry.name, sizeof(new_entry.name));
	if (in_file.eof())
	    break;
	in_file.getline(new_entry.address1, sizeof(new_entry.address1));
	in_file.getline(new_entry.address2, sizeof(new_entry.address2));
	in_file.getline(new_entry.city, sizeof(new_entry.city));
	in_file.getline(new_entry.state, sizeof(new_entry.state));
	in_file.getline(new_entry.zip, sizeof(new_entry.zip));
	new_entry.zip_number = atol(new_entry.zip);

	char entry_end[100];	// End of entry magic
	in_file.getline(entry_end, sizeof(entry_end));

	if (strcmp(entry_end, ENTRY_END) != 0) {
	    cerr << "Error: File corruption.\n";
	    return (-3);
	}
	mail_list[n_mail] = new mail;
	*mail_list[n_mail] = new_entry;
	++n_mail;
    }
    return (0);
}

/********************************************************
 * write_mail -- write out the mailing list.		*
 *							*
 * Returns						*
 *	0 -- no error.					*
 *	NZ -- error occurred, message written.		*
 ********************************************************/
int write_mail(
    const char file_name[]	// Name of the file to read
)
{
    ofstream out_file(file_name);	// File to write
    unsigned int i;			// Current entry

    if (out_file.bad()) {
	cerr << "Error: Could not open " << file_name << " for writing\n";
	return(-1);
    }

    out_file << MAIL_MAGIC << '\n';
    for (i = 0; i < n_mail; ++i) {
	out_file << mail_list[i]->name << '\n';
	out_file << mail_list[i]->address1 << '\n';
	out_file << mail_list[i]->address2 << '\n';
	out_file << mail_list[i]->city << '\n';
	out_file << mail_list[i]->state << '\n';
	out_file << mail_list[i]->zip << '\n';
	out_file << ENTRY_END << '\n';
    }
    return (0);
}
