/********************************************************
 * mail -- a mailing list program.			*
 *							*
 * Programming Problem: 14-4				*
 *							*
 * Usage:						*
 *	Run the program, follow the prompt		*
 *							*
 * Commands:						*
 *		A -- Add a new name to the list		*
 *		P -- Print the list in name order to 	*
 *			a file.				*
 *		Z -- Print the list in zip code order	*
 *			to a file			*
 *		Q -- Write out file and quit		*
 *							*
 * Limitations:						*
 *	Works for only small lists.			*
 *	Is designed for USA style addresses		*
 *	Does not take country into account.		*
 *	Does not handle Zip+4 addressing.		*
 *	Limited command set.				*
 *	Limited flexibility and options.		*
 *							*
 *	Major algorithm works, but the rest of the	*
 *	program needs the "fiddly bits" added.		*
 ********************************************************/
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <stdlib.h>

#include "mail.h"
#include "io.h"

mail *mail_list[NAME_MAX] = {0};	// Pointers to the mail data
unsigned int n_mail = 0;		// Number of entries in the mail data

const char MAIL_FILE[] = "mail.dat";	// Name of mail data file
/********************************************************
 * cmp_name -- Routine to compare two names for 	*
 *	qsort of a mailing list (sort by name)		*
 *							*
 * Returns result of the comparison.			*
 ********************************************************/
int cmp_name(
    const void *v1,		// Mail list entry #1
    const void *v2		// Mail list entry #2
) {
    const mail *m1 = *((mail **)v1);
    const mail *m2 = *((mail **)v2);

    return(strcmp(m1->name, m2->name));
}
/********************************************************
 * sort_name -- sort the mailing list by name		*
 ********************************************************/
inline void sort_name(void) {
    qsort(mail_list, n_mail, sizeof(mail*), cmp_name);
}
/********************************************************
 * cmp_zip -- Routine to compare two zip codes for 	*
 *	qsort of a mailing list (sort by zip)		*
 *							*
 * Returns result of the comparison.			*
 ********************************************************/
int cmp_zip(
    const void *v1,		// Mail list entry #1
    const void *v2		// Mail list entry #2
) {
    const mail *m1 = *((mail **)v1);
    const mail *m2 = *((mail **)v2);

    if (m1->zip_number < m2->zip_number) 
	return (-1);
    if (m1->zip_number > m2->zip_number) 
	return (1);
    return (0);
}
/********************************************************
 * sort_zip -- sort the mailing list by zip code	*
 ********************************************************/
inline void sort_zip(void) {
    qsort(mail_list, n_mail, sizeof(mail*), cmp_zip);
}

/********************************************************
 * print -- print labels to the file "print.out"	*
 ********************************************************/
void print(void)
{
    unsigned int index;	// Index of the item we are printing

    ofstream out_file("print.out");	// Output file
    if (out_file.bad()) {
	cerr << "Error: Could not write file: print.out\n";
	return;
    }

    for (index = 0; index < n_mail; ++index) {
	out_file << mail_list[index]->name << '\n';
	out_file << mail_list[index]->address1 << '\n';
	out_file << mail_list[index]->address2 << '\n';
	out_file << mail_list[index]->city << ", ";
	out_file << mail_list[index]->state << ' ';
	out_file << mail_list[index]->zip << '\n';
	out_file << '\n';
    }
}
/********************************************************
 * add_entry -- Add an entry to the list		*
 ********************************************************/
void add_entry(void)
{
    struct mail new_entry;	// Entry we are creating

    cout << "Enter name: ";
    cin.getline(new_entry.name, sizeof(new_entry.name));

    cout << "Enter address line 1: ";
    cin.getline(new_entry.address1, sizeof(new_entry.address1));

    cout << "Enter address line 2: ";
    cin.getline(new_entry.address2, sizeof(new_entry.address2));

    cout << "Enter city: ";
    cin.getline(new_entry.city, sizeof(new_entry.city));

    cout << "Enter state: ";
    cin.getline(new_entry.state, sizeof(new_entry.state));

    cout << "Enter zip code: ";
    cin.getline(new_entry.zip, sizeof(new_entry.zip));
    new_entry.zip_number = atol(new_entry.zip);

    mail_list[n_mail] = new mail;
    *mail_list[n_mail] = new_entry;
    ++n_mail;
}


main()
{
    read_mail(MAIL_FILE);

    char cmd;	// Command character

    while (1) {
	cout << "Command? ";
	cin >> cmd;
	cin.ignore(1, '\n');	// Flush the line

	switch (cmd) {
	    case 'A':
	    case 'a':
		add_entry();
		break;
	    case 'P':
	    case 'p':
		sort_name();
		print();
		break;
	    case 'Z':
	    case 'z':
		sort_zip();
		print();
		break;
	    case 'q':
	    case 'Q':
		write_mail(MAIL_FILE);
		exit(0);
	    default:
		cout << "Bad command: " << cmd << '\n';
		// Fall through
	    case '?':
	    case 'h':
	    case 'H':
		cout << "Command	Meaning\n";
		cout << "A	Add a new name to the list\n";
		cout << "P	Print labels in name order\n";
		cout << "Z	Print labels in zip code order\n";
		cout << "Q	Write file and quite\n";
		break;
	}
    }
}

